@@ -9,6 +9,9 @@ use LWP::UserAgent;
 my $baseurl = 'http://www.allopass.com/check/vf.php4';
 my $error   = '';
 
+use Exporter;
+@EXPORT=qw('allopass_check');
+
 =head1 NAME
 
 Billing::Allopass::Simple - A simple function for micropayment system from Allopass
@@ -25,14 +28,16 @@ Billing::Allopass::Simple - A simple function for micropayment system from Allop
   
 =head1 DESCRIPTION
 
-This function provides you a simple api to the allopass.com system.
+This module provides a simple API to the Allopass.com micropayment system.
 This alternative to Business::PhoneBill::Allopass justs performs access code checks.
 
 See I<http://www.allopass.com/index.php4?ADV=1508058> for more informations on their system and how it basically works.
 
 =head1 FUNCTIONS
 
-=item B<allopass_check> - Simply checks if a code has been recently validated for this document.
+=over 4
+
+=item B<allopass_check> - Checks if a code has been recently validated for this document.
 
     allopass_check($document_id, $code);
 
@@ -54,7 +59,7 @@ sub allopass_check {
     return _is_res_ok($res);
 }
 
-=item B<get_last_allopass_error> - Simply checks if a code has been recently validated for this document.
+=item B<get_last_allopass_error> - Returns last status string
 
     print get_last_allopass_error();
 
@@ -69,25 +74,27 @@ sub _is_res_ok {
     my($h, $c, $a)=split(/\n\n/, $res); chomp $c;
     if($res && $res!~/NOK/ && $res!~/ERR/ && $res!~/error/i && $c=~/OK/) {
         _set_error('Allopass Recall OK');
-        return 1;
+        return 0;
     }
     if ($c =~/NOK/) {
-        _set_error("Allopass.com says : This code is invalid")
+        return _set_error("Allopass.com says : This code is invalid")
     } elsif ($c =~/ERR/) {
-        _set_error("Allopass.com says : Invalid document id")
+        return _set_error("Allopass.com says : Invalid document id")
     } else {
         $res=~s/[\r\n]/ /g;
-        _set_error("Invalid Allopass.com response code : $res")
+        return _set_error("Invalid Allopass.com response code : $res")
     }
-    0;
+    1;
 }
 sub _set_error {
     $error=shift;
 }
 
+=back
+
 =head1 AUTHOR
 
-Bernard Nauwelaerts <bpn@it-development.be>
+Bernard Nauwelaerts <bpn#it-development%be>
 
 =head1 LICENSE
 
